import { ValiError } from '../../error';
import { executePipe, getCurrentPath, getErrorAndPipe } from '../../utils';
export function map(key, value, arg3, arg4) {
    // Get error and pipe argument
    const { error, pipe } = getErrorAndPipe(arg3, arg4);
    // Create and return map schema
    return {
        /**
         * The schema type.
         */
        schema: 'map',
        /**
         * The map key and value schema.
         */
        map: { key, value },
        /**
         * Whether it's async.
         */
        async: false,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        parse(input, info) {
            // Check type of input
            if (!(input instanceof Map)) {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'map',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create output and issues
            const output = new Map();
            const issues = [];
            // Parse each key and value by schema
            for (const [inputKey, inputValue] of input.entries()) {
                // Get current path
                const path = getCurrentPath(info, {
                    schema: 'map',
                    input,
                    key: inputKey,
                    value: inputValue,
                });
                // Parse key and get output
                let outputKey;
                try {
                    // Note: Output key is nested in array, so that also a falsy value
                    // further down can be recognized as valid value
                    outputKey = [key.parse(inputKey, { ...info, origin: 'key', path })];
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
                // Parse value and get output
                let outputValue;
                try {
                    // Note: Output value is nested in array, so that also a falsy value
                    // further down can be recognized as valid value
                    outputValue = [value.parse(inputValue, { ...info, path })];
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
                // Set entry if output key and value is valid
                if (outputKey && outputValue) {
                    output.set(outputKey[0], outputValue[0]);
                }
            }
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipe(output, pipe, { ...info, reason: 'map' });
        },
    };
}
