import { ValiError } from '../../error';
import { executePipeAsync, getCurrentPath, getErrorAndPipe } from '../../utils';
export function mapAsync(key, value, arg3, arg4) {
    // Get error and pipe argument
    const { error, pipe } = getErrorAndPipe(arg3, arg4);
    // Create and return async map schema
    return {
        /**
         * The schema type.
         */
        schema: 'map',
        /**
         * The map key and value schema.
         */
        map: { key, value },
        /**
         * Whether it's async.
         */
        async: true,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        async parse(input, info) {
            // Check type of input
            if (!(input instanceof Map)) {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'map',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create output and issues
            const output = new Map();
            const issues = [];
            // Parse each key and value by schema
            await Promise.all(Array.from(input.entries()).map(async ([inputKey, inputValue]) => {
                // Get current path
                const path = getCurrentPath(info, {
                    schema: 'map',
                    input,
                    key: inputKey,
                    value: inputValue,
                });
                const [outputKey, outputValue] = await Promise.all([
                    // Parse key and get output
                    (async () => {
                        try {
                            // Note: Output key is nested in array, so that also a falsy value
                            // further down can be recognized as valid value
                            return [
                                await key.parse(inputKey, { ...info, origin: 'key', path }),
                            ];
                            // Fill issues in case of an error
                        }
                        catch (error) {
                            issues.push(...error.issues);
                        }
                    })(),
                    // Parse value and get output
                    (async () => {
                        try {
                            // Note: Output value is nested in array, so that also a falsy value
                            // further down can be recognized as valid value
                            return [
                                await value.parse(inputValue, { ...info, path }),
                            ];
                            // Fill issues in case of an error
                        }
                        catch (error) {
                            issues.push(...error.issues);
                        }
                    })(),
                ]);
                // Set entry if output key and value is valid
                if (outputKey && outputValue) {
                    output.set(outputKey[0], outputValue[0]);
                }
            }));
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipeAsync(output, pipe, { ...info, reason: 'map' });
        },
    };
}
