import { ValiError } from '../../error';
import { executePipeAsync, getCurrentPath, getErrorAndPipe } from '../../utils';
export function objectAsync(object, arg2, arg3) {
    // Get error and pipe argument
    const { error, pipe } = getErrorAndPipe(arg2, arg3);
    // Create and return async object schema
    return {
        /**
         * The schema type.
         */
        schema: 'object',
        /**
         * The object schema.
         */
        object,
        /**
         * Whether it's async.
         */
        async: true,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        async parse(input, info) {
            // Check type of input
            if (!input ||
                typeof input !== 'object' ||
                input.toString() !== '[object Object]') {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'object',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create output and issues
            const output = {};
            const issues = [];
            // Parse schema of each key
            await Promise.all(Object.entries(object).map(async ([key, schema]) => {
                try {
                    const value = input[key];
                    output[key] = await schema.parse(value, {
                        ...info,
                        path: getCurrentPath(info, {
                            schema: 'object',
                            input,
                            key,
                            value,
                        }),
                    });
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
            }));
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipeAsync(output, pipe, {
                ...info,
                reason: 'object',
            });
        },
    };
}
