import { ValiError } from '../../error';
import { executePipe, getCurrentPath, getErrorAndPipe } from '../../utils';
import { string } from '../string';
export function record(arg1, arg2, arg3, arg4) {
    // Get key, value, error and pipe argument
    const { key, value, error, pipe } = (typeof arg2 === 'object' && !Array.isArray(arg2)
        ? { key: arg1, value: arg2, ...getErrorAndPipe(arg3, arg4) }
        : { key: string(), value: arg1, ...getErrorAndPipe(arg2, arg3) });
    // Create and return record schema
    return {
        /**
         * The schema type.
         */
        schema: 'record',
        /**
         * The record key and value schema.
         */
        record: { key, value },
        /**
         * Whether it's async.
         */
        async: false,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        parse(input, info) {
            // Check type of input
            if (!input ||
                typeof input !== 'object' ||
                input.toString() !== '[object Object]') {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'record',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create output and issues
            const output = {};
            const issues = [];
            // Parse each key and value by schema
            // Note: `Object.entries(...)` converts each key to a string
            Object.entries(input).forEach(([inputKey, inputValue]) => {
                // Get current path
                const path = getCurrentPath(info, {
                    schema: 'record',
                    input,
                    key: inputKey,
                    value: inputValue,
                });
                // Parse key and get output
                let outputKey;
                try {
                    outputKey = key.parse(inputKey, { ...info, origin: 'key', path });
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
                // Parse value and get output
                let outputValue;
                try {
                    // Note: Value is nested in array, so that also a falsy value further
                    // down can be recognized as valid value
                    outputValue = [value.parse(inputValue, { ...info, path })];
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
                // Set entry if output key and value is valid
                if (outputKey && outputValue) {
                    output[outputKey] = outputValue[0];
                }
            });
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipe(output, pipe, { ...info, reason: 'record' });
        },
    };
}
