import { ValiError } from '../../error';
import { executePipe, getCurrentPath, getErrorAndPipe } from '../../utils';
export function set(value, arg2, arg3) {
    // Get error and pipe argument
    const { error, pipe } = getErrorAndPipe(arg2, arg3);
    // Create and return set schema
    return {
        /**
         * The schema type.
         */
        schema: 'set',
        /**
         * The set value schema.
         */
        set: { value },
        /**
         * Whether it's async.
         */
        async: false,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        parse(input, info) {
            // Check type of input
            if (!(input instanceof Set)) {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'set',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create index, output and issues
            let index = 0;
            const output = new Set();
            const issues = [];
            // Parse each value by schema
            for (const inputValue of input) {
                try {
                    output.add(value.parse(inputValue, {
                        ...info,
                        path: getCurrentPath(info, {
                            schema: 'set',
                            input,
                            key: index++,
                            value: inputValue,
                        }),
                    }));
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
            }
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipe(output, pipe, { ...info, reason: 'set' });
        },
    };
}
