import { ValiError } from '../../error';
import { executePipeAsync, getCurrentPath, getErrorAndPipe } from '../../utils';
export function tupleAsync(items, arg2, arg3, arg4) {
    // Get rest, error and pipe argument
    const { rest, error, pipe } = (typeof arg2 === 'object' && !Array.isArray(arg2)
        ? { rest: arg2, ...getErrorAndPipe(arg3, arg4) }
        : getErrorAndPipe(arg2, arg3));
    // Create and return async tuple schema
    return {
        /**
         * The schema type.
         */
        schema: 'tuple',
        /**
         * The tuple items and rest schema.
         */
        tuple: { items, rest },
        /**
         * Whether it's async.
         */
        async: true,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        async parse(input, info) {
            // Check type of input
            if (!Array.isArray(input) ||
                (!rest && items.length !== input.length) ||
                (rest && items.length > input.length)) {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'tuple',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        ...info,
                    },
                ]);
            }
            // Create output and issues
            const output = [];
            const issues = [];
            await Promise.all([
                // Parse schema of each tuple item
                Promise.all(items.map(async (schema, index) => {
                    try {
                        const value = input[index];
                        output[index] = await schema.parse(value, {
                            ...info,
                            path: getCurrentPath(info, {
                                schema: 'tuple',
                                input: input,
                                key: index,
                                value,
                            }),
                        });
                        // Fill issues in case of an error
                    }
                    catch (error) {
                        issues.push(...error.issues);
                    }
                })),
                // If necessary parse schema of each rest item
                rest &&
                    Promise.all(input.slice(items.length).map(async (value, index) => {
                        try {
                            const tupleIndex = items.length + index;
                            output[tupleIndex] = await rest.parse(value, {
                                ...info,
                                path: getCurrentPath(info, {
                                    schema: 'tuple',
                                    input: input,
                                    key: tupleIndex,
                                    value,
                                }),
                            });
                            // Fill issues in case of an error
                        }
                        catch (error) {
                            issues.push(...error.issues);
                        }
                    })),
            ]);
            // Throw error if there are issues
            if (issues.length) {
                throw new ValiError(issues);
            }
            // Execute pipe and return output
            return executePipeAsync(output, pipe, {
                ...info,
                reason: 'tuple',
            });
        },
    };
}
