import { ValiError } from '../../error';
/**
 * Creates a union schema.
 *
 * @param union The union schema.
 * @param error The error message.
 *
 * @returns A union schema.
 */
export function union(union, error) {
    return {
        /**
         * The schema type.
         */
        schema: 'union',
        /**
         * The union schema.
         */
        union,
        /**
         * Whether it's async.
         */
        async: false,
        /**
         * Parses unknown input based on its schema.
         *
         * @param input The input to be parsed.
         * @param info The parse info.
         *
         * @returns The parsed output.
         */
        parse(input, info) {
            // Create output and issues
            let output;
            const issues = [];
            // Parse schema of each option
            for (const schema of union) {
                try {
                    // Note: Output is nested in array, so that also a falsy value
                    // further down can be recognized as valid value
                    output = [schema.parse(input, info)];
                    break;
                    // Fill issues in case of an error
                }
                catch (error) {
                    issues.push(...error.issues);
                }
            }
            // Throw error if every schema failed
            if (!output) {
                throw new ValiError([
                    {
                        reason: 'type',
                        validation: 'union',
                        origin: 'value',
                        message: error || 'Invalid type',
                        input,
                        issues: issues,
                        ...info,
                    },
                ]);
            }
            // Otherwise return parsed output
            return output[0];
        },
    };
}
